/*
 *
 *   MiniPad - a trivial java editor written in java. 
 *   it helps you to develop java and c programs in the simpliest way possible
 *   providing the minimum set of functions needed and the shortest way to get them
 *   (like a big, clear and coloured button, a menu item and a keybord shortcut).
 *   It acts as a frontend for javac and gcc.
 *
 *   Require JDK 1.2
 *           
 *   Copyright (C) 1999  Andrea Gini
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Send questions or suggestions to ginian@lib.disco.unimib.it
 *
 */

import java.util.*;

import java.io.*;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
/**
   E' il componente centrale del sistema.
   Il suo scopo principale e' quello di avviare il sistema e di gestire lo svolgimento delle principali Action.
   Non possiede uno stato interno, ma ha una visione completa dello stato del sistema, contenuto nei componenti Compiler, RuntimeEnv, MPDocument
 */
public abstract class AbstractMiniPad
{
  private MPGui _gui;
  private DefaultEditorKit _editorKit;
  private MPVirtualMachine _virtualMachine;
  private MPCompiler _compiler;
  private Process _runningCompiler;
  private Vector _processes;
  private Action[] _actions =
  {
    new NewAction(),
    new OpenAction(),
    new SaveAction(),
    new SaveAsAction(),
    new ExitAction(),
    new CompileAction(),
    new RunAction(),
    new StopAction(),
  };

  private JFileChooser _chooser;
  /**
   * Il costruttore avvia il sistema. Le sottoclassi concrete
   * non hanno bisogno di sovrascriverlo, poiche' per
   * creare i componenti dipendenti dall'implementazione
   * vengono invocati i metodi createDocument(), createCompiler()
   * e createVirtualMachine()
   */
  public AbstractMiniPad()
  {
    _virtualMachine=createVirtualMachine();
    _compiler=createCompiler();
    _gui=new MPGui(_actions);

    _editorKit = new DefaultEditorKit();
    MPDocument doc = createDocument();
    _gui.setDocument(doc);
    _processes=new Vector();
  }
    /**
       Viene invocato dal costruttore all'atto di creare la
       VirtualMachine. Le sottoclassi di MiniPad che vogliano
       gestire un tipo diverso di VM devono sovrascrivere questo
       metodo.
     */
    protected abstract MPVirtualMachine createVirtualMachine();
    /**
       Viene invocato dal costruttore all'atto di creare il
       compilatore. Le sottoclassi di MiniPad che vogliano
       gestire un tipo diverso di cmpilatore devono sovrascrivere questo metodo.
     */
    protected abstract MPCompiler createCompiler();
    /**
       Viene invocato dal costruttore all'atto di creare il Document.
       Le sottoclassi di MiniPad che vogliano gestire un tipo
       diverso di VM devono sovrascrivere questo metodo.
       @roseuid 38BD474001E0
     */
    protected abstract MPDocument createDocument();
 
    /**
       Chiede all'utente se vuole salvare le modifiche, e quindi crea un nuovo documento.
       @roseuid 38A166EE0046
     */
    public void newDocument()
    {
      try
      {
        closeDocument();
        MPDocument newDocument = createDocument();
        _gui.setDocument(newDocument);
      }
      catch(AbortException ae)
      {
        return;
      }
    }
  public void closeDocument() throws AbortException
  {
    JFrame frame = _gui.getFrame();
    MPDocument doc = _gui.getDocument();
    String warning;      
      
    if(doc!=null)
      warning = "Save Changes on "+doc.getDocumentName()+" before closing?";
    else
      warning = "Save Changes before closing?";
        
    int opt = JOptionPane.showConfirmDialog(frame,warning ,"MiniPad",JOptionPane.YES_NO_CANCEL_OPTION);
  
    if(opt==JOptionPane.CANCEL_OPTION)
      throw new AbortException();   
    else if(opt==JOptionPane.NO_OPTION)
      return;
    else if(opt==JOptionPane.YES_OPTION)
    {
      save();  // anche questa chiamata puo' lanciare una AbortException
      return;
    }
  }
  /**
     Apre un documento presente sul disco. All'utente viene presentato un JFileChooser per scegliere il file
   */
  public void openDocument()
  {
    try
    {
      closeDocument();
    }
    catch(AbortException ae)
    {
      return; // operazione abortita
    }
  
    JFrame frame = _gui.getFrame();
    if(_chooser==null)
      _chooser = new JFileChooser();
    int returnVal = _chooser.showOpenDialog(frame);

    if(returnVal == JFileChooser.APPROVE_OPTION)
    {
      File f = _chooser.getSelectedFile();

      MPDocument doc = createDocument();
      doc.setFile(f);
      _gui.setDocument(doc);
      
      try
      {
        Reader in = new FileReader(f);
        _editorKit.read(in,doc,0);
        in.close();
      }
      catch(BadLocationException ble)
      {
         System.err.println(ble.toString());
      }
      catch(IOException ioe)
      {
        System.err.println(ioe.toString());
      }      
    }
  }
  /**
     Salva su file il contenuto del documento. Se il documento non e' associato ad un File, viene chiamato saveAs().
    */
    public void save() throws AbortException
    {
      MPDocument doc = _gui.getDocument();
      File f = doc.getFile();
      if(f==null)
        saveAs();  // da qui puo' provenire una AbortException
      else
        try
        {
          /*
           * crea file di backup
           */
          if(f.exists() /* &&  _doc.isBackupEnabled() */)
          {
            File bak = new File(f.getAbsolutePath()+".bak");
            File old = f;
            if(bak.exists())
            bak.delete();
            f.renameTo(new File(f.getAbsolutePath()+".bak"));
            f = old;
          }

         Writer out = new FileWriter(f);      
         _editorKit.write(out,doc,0,doc.getLength());
         out.close();
        }
        catch(IOException e)
        {
          System.err.println(e.toString());
        }
        catch(BadLocationException be)
        {
          System.err.println(be.toString());
        }      
     }
    
    /**
       chiede all'utente (con un JFileChooser) di inserire un nuovo nome,e quindi chiama save()
       @roseuid 38A16D2401B8
     */
    public void saveAs() throws AbortException
    {
      MPDocument doc = _gui.getDocument();

      File f;
      f = doc.getFile();
      JFrame frame = _gui.getFrame();
      
      if(_chooser==null)
        _chooser = new JFileChooser();
      
      if(f!=null)
        _chooser.setSelectedFile(f);
        
      int returnVal = _chooser.showSaveDialog(frame);
      if(returnVal == JFileChooser.APPROVE_OPTION)
      {
        doc.setFile(_chooser.getSelectedFile());
        save();
      }
      else if(returnVal == JFileChooser.CANCEL_OPTION)
        throw new AbortException();
    }
    
    /**
       Mostra una finestra per chiedere all'utente se vuole salvare le modifiche, e in caso affermativo, esce dal programma.
     */
    public void exit() throws AbortException
    {
      try
      {
        closeDocument();  // da qui puo' essere lanciata una AbortException
        System.exit(0);
      }
      catch(AbortException ae)
      {
        return;
      }
    }
    
    /**
       Avvia la compilazione del sorgente che si sta' editando. Prima pero' chiama save().
     */
    public void compile()
    {
      try
      {
        save();
      }
      catch(AbortException ae)
      {
        return;
      }    
      MPDocument doc = _gui.getDocument();
      if(_runningCompiler!=null)      // uccide il precedente processo di compilazione
        _runningCompiler.destroy();
      
      _runningCompiler=_compiler.compile(doc);
      Console c = _gui.getConsole();
      c.bindOutput(_runningCompiler,"Compiler");
    }
    
    /**
       Avvia l'applicazione associata al sorgente dell'editor.
     */
    public void runProgram()
    {
      MPDocument doc = _gui.getDocument();
      
      Process p = _virtualMachine.exec(doc);
      Console c = _gui.getConsole();      
      c.bindOutput(p,"Process "+doc.getDocumentName());
      _processes.add(p);
    }
    
    /**
       Uccide i processi avviati.
       @roseuid 38A16D820212
     */
    public void stopProgram()
    {
      int size = _processes.size();
      if (size != 0)
        for(int i=0;i<size;i++)
        {
          Process p = (Process)_processes.elementAt(i);
          if(p != null)
            p.destroy();
        }
      _processes.clear();      
    }


public class NewAction extends MPAbstractAction
{
public NewAction()
{
  super(NEW);
}
/**
 * Mostra una finestra che permette di inserire il nome del documento e il tipo.
 * Prova chiamare il metodo newDocument su JiniPad e mostra un messaggio di errore
 * se io documento risulta gia' aperto.
 */
public void actionPerformed(ActionEvent e)
{
  newDocument();
}

}
public class OpenAction extends MPAbstractAction
{
public OpenAction()
{
  super(OPEN);
}
public void actionPerformed(ActionEvent e)
{
  openDocument();
}
}
public class SaveAction extends MPAbstractAction
{
public SaveAction()
{
  super(SAVE);
}
public void actionPerformed(ActionEvent e)
{
  try
  {
    save();
  }
  catch(AbortException ae)
  {}
}
}
public class SaveAsAction extends MPAbstractAction
{
public SaveAsAction()
{
  super(SAVE_AS);
}
public void actionPerformed(ActionEvent e)
{
  try
  {
    saveAs();
  }
  catch(AbortException ae)
  {}
}
}
public class ExitAction extends MPAbstractAction
{
public ExitAction()
{
  super(EXIT);
}
public void actionPerformed(ActionEvent e)
{
  try
  {
    exit();
  }
  catch(AbortException ae)
  {}
}
}
public class CompileAction extends MPAbstractAction
{
public CompileAction()
{
  super(COMPILE);
}
public void actionPerformed(ActionEvent e)
{
    compile();
}
}
public class RunAction extends MPAbstractAction
{
public RunAction()
{
  super(RUN);
}
public void actionPerformed(ActionEvent e)
{
  runProgram();
}
}
public class StopAction extends MPAbstractAction
{
public StopAction()
{
  super(STOP);
}
public void actionPerformed(ActionEvent e)
{
  stopProgram();
}
}

}
